﻿
using System.ComponentModel.DataAnnotations;
namespace Library.Services {
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.DataAnnotations;
    using System.Linq;
    using System.Web.Ria;
    using System.Web.Ria.Data;
    using System.Web.DomainServices;
    using System.Data.Linq;
    using System.Web.DomainServices.LinqToSql;
    using Library.DataModel;
    using System.Data.Linq.SqlClient;


    // Implements application logic using the LibraryDBDataContext context.
    // TODO: Add your application logic to these methods or in additional methods.
    [EnableClientAccess()]
    [RequiresAuthentication]
    public class LibraryManagementService : LinqToSqlDomainService<LibraryDBDataContext> {

        public IQueryable<Book> GetBooks() {
            return from b in Context.Books
                   where b.Missing == false
                   orderby b.PublishDate descending
                   select b;
        }

        public IQueryable<Book> SearchBooks(string searchText) {
            if (String.IsNullOrEmpty(searchText)) {
                return GetBooks();
            }

            string searchPattern = "%" + searchText + "%";

            return from b in Context.Books
                   where b.Missing == false &&
                         (SqlMethods.Like(b.Title, searchPattern) ||
                          SqlMethods.Like(b.Author, searchPattern) ||
                          SqlMethods.Like(b.Description, searchPattern))
                   orderby b.PublishDate descending
                   select b;
        }

        public void UpdateBook(Book book) {
            Context.Books.Attach(book, ChangeSet.GetOriginal(book));

            // TODO: Email notifications... and other business logic
        }

        public IEnumerable<BookYearStat> GetStats() {
            var bookGroups =
                Context.Books.Where(b => b.Missing == false).GroupBy(b => b.PublishDate.Year);

            List<BookYearStat> stats = new List<BookYearStat>();
            foreach (var group in bookGroups) {
                BookYearStat stat = new BookYearStat() {
                    Year = group.Key,
                    BookCount = group.Count()
                };

                stats.Add(stat);
            }

            return stats;
        }

        public void Checkout(Book book) {
            Book existingBook = Context.Books.Where(b => b.BookID == book.BookID).FirstOrDefault();
            if ((existingBook == null) || existingBook.Missing) {
                throw new ValidationException("The specified book does not exist.");
            }
            if (existingBook.OnShelf == false) {
                throw new ValidationException("The specified book is already checked out.");
            }

            existingBook.OnShelf = false;
            book.OnShelf = false;
        }
    }
}

namespace Library.DataModel {

    public class BookYearStat {

        [Key]
        public int Year {
            get;
            set;
        }

        public int BookCount {
            get;
            set;
        }
    }
}


