﻿// Bookshelf.cs
//

using System;
using System.Linq;
using System.Web.Ria;
using Microsoft.Azure.DomainServices;

namespace Library.Web {

    [EnableClientAccess]
    public class Bookshelf : AzureDomainService<BooksDBDataContext> {

        private BooksDBDataContext _dataContext;

        #region Azure Stuff
        protected override string GetPartitionKey() {
            return "Bookshelf";
        }

        protected override BooksDBDataContext CreateDataContext() {
            if (_dataContext != null) {
                return _dataContext;
            }

            _dataContext = base.CreateDataContext();
            _dataContext.RetryCount = 2;

            return _dataContext;
        }
        #endregion

        #region CRUD - Query, Insert and Update
        public IQueryable<Book> GetBooks() {
            return DataContext.Books;
        }

        public void AddBook(Book book) {
            book.RowKey = Guid.NewGuid().ToString();
            DataContext.Books.InsertOnSubmit(book);
        }

        public void UpdateBook(Book currentBook) {
            DataContext.Books.Attach(currentBook, ChangeSet.GetOriginal(currentBook), GetETag(currentBook));
        }
        #endregion
    }
}
